
function [Run] = INITIALISE_BGM_D(DATA_ALL, DAG, steps, lambda)

[n_plus, m]= size(DATA_ALL{1});

n_nodes = length(DATA_ALL);

%Initialisation (may be changed)
k = 1; % -> k_break = 0 that is: one component <-> no breakpoint 
breakpoints = [];

[vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);

for node_i=1:n_nodes
        
    data = DATA_ALL{node_i};
        
    for i=1:k      
      DATA{node_i}{i} = data(:,find(vector==i));
    end
    
end

[log_score] = COMPUTE_LOG_SCORE_OF_DAG_k(DATA, DAG, breakpoints, lambda, m);


for i=1:steps+1
Run.dag{i}         = 0;
Run.vector{i}      = 0;
Run.components(i)  = 0;
Run.breakpoints{i} = 0;
Run.Log_Scores(i)  = 0;

Run.Simulation(i)  = 0;

Run.A_BIRTH(i)     = 0;
Run.A_DEATH(i)     = 0;
Run.A_ALLO(i)      = 0;
Run.A_EDGE(i)      = 0;

Run.R_BIRTH(i)     = 0;
Run.R_DEATH(i)     = 0;
Run.R_ALLO(i)      = 0;
Run.R_EDGE(i)      = 0;
end    
    
    
Run.dag{1}         = DAG;
Run.vector{1}      = vector;
Run.components(1)  = k;
Run.Log_Scores(1)  = log_score;
Run.breakpoints{1} = breakpoints;

Run.steps(1)       = 1;


return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m)

vector = ones(1,m);

if isempty(breakpoints)
    vector = ones(1,m);
else
    breakpoints_add = [1,breakpoints,m];
    
    for i=2:length(breakpoints_add)
        vector(ceil(breakpoints_add(i-1)):floor(breakpoints_add(i))) = i-1;        
    end

end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [log_score] = COMPUTE_LOG_SCORE_OF_DAG_k(DATA, DAG, breakpoints, lambda,m)

% Interpretation of DATA:
% DATA{node_i}{1} -> data set of observations allocated to k=1 
% for scoring node_i
%
% DATA{node_i}{2} -> data set of observations allocated to k=2
% for scoring node_i


% The hyperparameters:
global T_0;
global v;
global alpha;
global my_0;
global Prior;

k = length(DATA{1});      % k is the number of mixture components

k_break = k-1;

log_prob_k = log(poisspdf(k_break,lambda));

[n_plus_thrash, n_nodes] = size(DAG);
clear n_plus_thrash;

log_prob_break = 0;

if isempty(breakpoints)   
    %log_prob_break = 0 is okay
else
    log_prob_break = log(prod(1:(2*k_break+1))) + (-1)*k_break*log(m-2);
    % interval [2,m] -> length = m-2
    breakpoints = [2,breakpoints,m];
    
    for i=2:length(breakpoints)
       log_prob_break = log_prob_break + log((breakpoints(i)-breakpoints(i-1))) - log((m-2)); 
    end        
end


%Initialisation:
log_score = log_prob_k + log_prob_break;

for node=1:n_nodes
    log_score = log_score + Prior(length(find(DAG(:,node)))+1);
end


% Compute the score for each of the k mixture components:
for component=1:k
    
    log_score_component          = 0;
    [n_plus, n_obs_in_component] = size(DATA{1}{component});

    if(n_obs_in_component==0)
        
    log_score_component = 0; 
    
    else
        
        % Compute the score for the current mixture component:
        for node = 1:n_nodes
            
            data = DATA{node}{component};
    
            % Compute T_m for the current combination of node and
            % component:
            S_m = zeros(n_plus,n_plus);
    
                for i=1:n_obs_in_component    
                    S_m = S_m + (data(:,i) - mean(data')') * (data(:,i) - mean(data')')';
                end
  
            T_m = T_0 + S_m + (v*n_obs_in_component)/(v+n_obs_in_component) * (my_0 - mean(data')') * (my_0 - mean(data')')';  
            
            parents_of_node  = find(DAG(:,node));
            parents_and_node = [parents_of_node; n_plus];
           
            [log_score_local_node_nominator]   = Gauss_Score_complete(length(parents_and_node), n_obs_in_component, v, alpha, T_0(parents_and_node,parents_and_node), T_m(parents_and_node,parents_and_node));
  
            [log_score_local_node_denominator] = Gauss_Score_complete(length(parents_of_node),  n_obs_in_component, v, alpha, T_0(parents_of_node,parents_of_node),   T_m(parents_of_node,parents_of_node));
    
            log_score_component = log_score_component + (log_score_local_node_nominator - log_score_local_node_denominator);   
        end 
    end
    
    log_score = log_score + log_score_component;
   
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [log_Score_i] = Gauss_Score_complete(n_nodes_in_sub, n_obs, v, alpha, T_0_sub, T_m_sub)

if n_nodes_in_sub==0
log_Score_i = 0;
else
      
sum_1 = (-1)*(n_nodes_in_sub)*(n_obs)/2 * log(2*pi) + (n_nodes_in_sub/2) * log(v/(v+n_obs));

sum_3 = (alpha/2) * log(det(T_0_sub)) + (-1)*(alpha+n_obs)/2 * log(det(T_m_sub));

log_value_a = (alpha         *n_nodes_in_sub/2)*log(2) + (n_nodes_in_sub*(n_nodes_in_sub-1)/4)*log(pi);
log_value_b = ((alpha+n_obs) *n_nodes_in_sub/2)*log(2) + (n_nodes_in_sub*(n_nodes_in_sub-1)/4)*log(pi);
 
for i=1:n_nodes_in_sub
    log_value_a = log_value_a + gammaln(( alpha   +1-i)/2);
    log_value_b = log_value_b + gammaln(((alpha+n_obs)+1-i)/2);
end
   
log_Score_i = sum_1 + (-1)*(log_value_a - log_value_b) + sum_3; 

end

return 
    