

function [Run] = PROC_BGM_D(data,steps,step_iterations,k_max,lambda)

global DATA_ALL;

[DATA_ALL] = SHIFT_DATA(data);

global T_0;
global my_0;
global v;
global alpha;
global Prior;

[T_0, my_0, v, alpha, Prior] = SET_HYPERPARAMETERS(DATA_ALL);

n_nodes          = length(DATA_ALL);

[proposal_table] = PRECOMPUTE_C(lambda,k_max);

DAG = zeros(n_nodes,n_nodes);

[Run] = INITIALISE_BGM_D(DATA_ALL, DAG, steps, lambda);
[Run] = START_BGM_D(DATA_ALL, steps, step_iterations, proposal_table, k_max, lambda, Run);

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [DATA_SHIFTED] = SHIFT_DATA(data)

[n, t] = size(data);
n_plus = n+1;

for node_i=1:n

            % For each variable X_i consider
            % data_1[X_1(t-1),...,X_(n)(t-1),...,X_1(t-1),...,X_(n)(t-1), X_(i)(t)] 
            obs                  = 1:(t-1);
            data_new             = zeros(n_plus,t-1);      
            data_new(1:n,obs)    = data(1:n,1:(t-1));       
            data_new(n+1,obs)    = data(node_i,2:t);   
            DATA_SHIFTED{node_i} = data_new;      
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [T_0, my_0, v, alpha, Prior] = SET_HYPERPARAMETERS(DATA_ALL)

[n_plus, n_obs] = size(DATA_ALL{1});

n_nodes = n_plus-1;

% VALUES MAY BE MODIDIED BY A USER: v, alpha, my_0, and B 
Prior = zeros(n_nodes+1,1); 
v     = 1;
alpha = n_plus+2;
my_0  = 0 * ones(n_plus,1);
v_vec = 1 * ones(n_plus,1); 
B     = zeros(n_plus,n_plus);

% COMPUTE PRECISION-MATRIX W
W =  1/v_vec(1);

for i = 1:(n_plus-1)
    b_vec = B(1:i,(i+1));
    W = [W + (b_vec * b_vec')/v_vec(i+1) , (-1)*b_vec/v_vec(i+1) ; (-1)*b_vec'/v_vec(i+1), 1/v_vec(i+1)];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE T_0: 
T_0   = (v*(alpha-n_plus-1))/(v+1) * inv(W);   

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [proposal_table] = PRECOMPUTE_C(lambda,k_max)

b = zeros(k_max,1);
d = zeros(k_max,1);

for k=1:(k_max-1)  
    b(k,1) = min([1,lambda/(k+1)]);   
end

for k=2:k_max   
    d(k,1) = min([1,(k+1)/lambda]);   
end

c_value = min(0.4./(b+d));

proposal_table = [0.5 * ones(k_max,1), c_value * b, c_value * d, 1 - c_value * (b + d) - 0.5 * ones(k_max,1) ];

proposal_table(1,1) = proposal_table(1,1) + proposal_table(1,4);
proposal_table(1,4) = 0;

return










