

function [Run] = START_BGM_D(DATA_ALL, steps, step_iterations, proposal_table, k_max, lambda, Run)

% Some defaults:

fan_in = 3; % maximal size of parent-sets                      

last           = Run.steps(1);          % index of last sampled dags

DAG            = Run.dag{last};         % the current DAG
vector         = Run.vector{last};      % the current allocation vector
log_score      = Run.Log_Scores(last);  % the current log score
k              = Run.components(last);  % the current number of components
breakpoints    = Run.breakpoints{last}; % the current set of breakpoints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output on screen: 
fprintf('\n###########################################################\n')
fprintf('The MCMC-simulation has been started \n')
fprintf('###########################################################\n')
fprintf('Log-likelihood-value of the initial graph: %1.5f \n',log_score)
fprintf('###########################################################\n\n')

%%% Start of the MCMC-simulation

for i = last:(steps+1)

[DAG, log_score, k, vector, breakpoints, A_BIRTH, A_DEATH, A_ALLO, A_EDGE, R_BIRTH, R_DEATH, R_ALLO, R_EDGE] = MCMC_INNER(step_iterations, DAG, fan_in, log_score, k, vector, DATA_ALL, k_max, lambda, proposal_table, breakpoints);

Run.dag{i+1}          = DAG;
Run.vector{i+1}       = vector;
Run.components(i+1)   = k;
Run.breakpoints{i+1}  = breakpoints;
Run.Log_Scores(i+1)   = log_score;

Run.Simulation(i+1)   = Run.Simulation(i) + step_iterations;

Run.A_BIRTH(i+1)      = Run.A_BIRTH(i)      + A_BIRTH;
Run.A_DEATH(i+1)      = Run.A_DEATH(i)      + A_DEATH;
Run.A_ALLO(i+1)       = Run.A_ALLO(i)       + A_ALLO;
Run.A_EDGE(i+1)       = Run.A_EDGE(i)       + A_EDGE;

Run.R_BIRTH(i+1)      = Run.R_BIRTH(i)      + R_BIRTH;
Run.R_DEATH(i+1)      = Run.R_DEATH(i)      + R_DEATH;
Run.R_ALLO(i+1)       = Run.R_ALLO(i)       + R_ALLO;
Run.R_EDGE(i+1)       = Run.R_EDGE(i)       + R_EDGE;

Run.steps(1)          = i+1;

end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [DAG, log_score, k, vector, breakpoints, a_BIRTH, a_DEATH, a_ALLO, a_EDGE, r_BIRTH, r_DEATH, r_ALLO, r_EDGE] = MCMC_INNER(step_iterations, DAG, fan_in, log_score, k, vector, DATA_ALL, k_max, lambda, proposal_table, breakpoints)

[n_plus, m]  = size(DATA_ALL{1}); 
[n_parents, n_nodes] = size(DAG);

a_BIRTH = 0;
a_DEATH = 0; 
a_ALLO  = 0; 
a_EDGE  = 0;

r_BIRTH = 0;
r_DEATH = 0; 
r_ALLO  = 0;
r_EDGE  = 0;

%%%%%% Start the MCMC simulation:

for t=1:step_iterations

    x = rand;
    
    DATA     = [];
    DATA_NEW = [];
    
    p_0 = sum(proposal_table(k,1:1)); % structure MCMC
    p_1 = sum(proposal_table(k,1:2)); % Birth Move
    p_2 = sum(proposal_table(k,1:3)); % Death Move
    % else                            % Reallocation Move

    % note that the probabilities for k_break=0 are stored in the first row 
    
    if(x<=p_0) % PERFORM A STRUCTURE-MCMC MOVE
            
        nodes_indicis = randperm(n_nodes);
        child_node    = nodes_indicis(1);

        old_parents = DAG(:,child_node);
        new_parents = old_parents;
            
        parents_card_old = sum(old_parents);
                
        parents_indicis_old = find(old_parents);
                
                if (parents_card_old<fan_in);
            
                        neighbours_old = n_parents; 
                       
                        indicis = randperm(n_parents);
                        x = indicis(1);
                    
                        parent_index = x; % delete or add this parent node
            
                        new_parents(parent_index,1) = 1 - new_parents(parent_index,1); 
                
                
                else % elseif (parent_card_old==fan_in)
                             
                    x = randperm(fan_in);
                    x = x(1);
                
                    parent_index = parents_indicis_old(x); % delete this parent node
                
                    new_parents(parent_index,1) = 0;
                
                    neighbours_old = parents_card_old; % = fan_in
                end
                
                    parents_card_new    = sum(new_parents);
                
                if (parents_card_new<fan_in)         
                    neighbours_new = n_parents; % = n_parents-1
                else
                    neighbours_new = parents_card_new; % = fan_in
                end
             
                DAG_NEW = DAG;
                DAG_NEW(:,child_node) = new_parents;
                
                for node_i=1:n_nodes        
                    data = DATA_ALL{node_i};
                    for i=1:k      
                        DATA{node_i}{i} = data(:,find(vector==i));
                    end
                end
                          
                local_score_new = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA, DAG_NEW);
                local_score_old = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA, DAG);
            
                A = exp(local_score_new - local_score_old + log(neighbours_old) - log(neighbours_new));
        

            u = rand(1); 
 
            if u > min(1,A) % then reject the move:
            % DAG stays the same
            log_score_diff = 0;
            r_EDGE           = r_EDGE + 1;
            else % accept the move:

            DAG = DAG_NEW; 
  
            log_score_diff = local_score_new - local_score_old; 
            a_EDGE = a_EDGE + 1;
            end
         
            clear DATA;
            
            log_score = log_score + log_score_diff;
        
    elseif(x<=p_1) % PERFORM A BREAKPOINT BIRTH MOVE
    
            k_new = 1 + rand(1) * (m-1);
     
            breakpoints_add = [1,breakpoints,m]; % m = original m minus 1
         
            
            k_lower = find(breakpoints_add<k_new);
            k_minus = breakpoints_add(k_lower(end)); % greatest breakpoint (lower than k_new)
            
            k_higher = find(breakpoints_add>k_new);
            k_plus   = breakpoints_add(k_higher(1)); % smallest breakpoint (greater than k_new)

            k_break = k-1;
            
            log_prob_ratio =   log(m-1)-log(k_break+1) + log(2*(k_break+1)*(2*k_break+3)) - 2* log(m-1) + log((k_new - k_minus) * (k_plus - k_new)) - log(k_plus - k_minus);
     
            breakpoints_new = sort([breakpoints,k_new]);
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);
            
             for node_i=1:n_nodes
                    data = DATA_ALL{node_i};
                    for i=1:k      
                        DATA{node_i}{i} = data(:,find(vector==i));
                    end
             end
            
             local_score_old = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA, DAG);
             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
             [vector_new] = BREAKPOINTS_TO_VECTOR(breakpoints_new,m);
             
             for node_i=1:n_nodes
        
                    data = DATA_ALL{node_i};
        
                    for i=1:k+1      
                        DATA_NEW{node_i}{i} = data(:,find(vector_new==i));
                    end
    
             end
             
             local_score_new = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA_NEW, DAG);
             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
             A = exp(local_score_new - local_score_old + log_prob_ratio);
        
             u = rand(1); 
 
             if u > min(1,A) % then reject the move:
             r_BIRTH           = r_BIRTH + 1;
             else % accept the move:
            
             k = k+1;
           
             breakpoints = breakpoints_new;
             
             [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);
                       
             log_score = COMPUTE_LOG_SCORE_OF_DAG_k(DATA_NEW, DAG,     breakpoints, lambda,m);
             a_BIRTH   = a_BIRTH + 1;
             end
         
             clear DATA;
             clear DATA_NEW;

    elseif(x<=p_2)  % PERFORM A BREAKPOINT DEATH MOVE
               
        k_break = k-1;
        
        index = randperm(length(breakpoints));
        index = index(1);
        
        breakpoints_new = [breakpoints(1:(index-1)),breakpoints(index+1:end)];
          
        breakpoints_new_add = [1,breakpoints_new,m];
        
        k_away = breakpoints(index);
        
        k_lower = find(breakpoints_new_add<k_away);
        k_minus = breakpoints_new_add(k_lower(end)); % greatest breakpoint (lower than k_away)
            
        k_higher = find(breakpoints_new_add>k_away);
        k_plus   = breakpoints_new_add(k_higher(1)); % smallest breakpoint (greater than k_new)
        
        log_prob_ratio = log(k_break) - log(m-1) + 2* log(m-1) - log(2*(k_break+1)*(2*k_break)) + log(k_plus - k_minus)  - log((k_away - k_minus) * (k_plus - k_away));
         

         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);
            
             for node_i=1:n_nodes
                    data = DATA_ALL{node_i};
                    for i=1:k      
                        DATA{node_i}{i} = data(:,find(vector==i));
                    end
             end
            
          local_score_old = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA, DAG);
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          [vector_new] = BREAKPOINTS_TO_VECTOR(breakpoints_new,m);
             
          for node_i=1:n_nodes     
                 data = DATA_ALL{node_i};        
                    for i=1:k+1      
                        DATA_NEW{node_i}{i} = data(:,find(vector_new==i));
                    end
          end
             
          local_score_new = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA_NEW, DAG);
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
          A = exp(local_score_new - local_score_old + log_prob_ratio);
        
          u = rand(1); 
 
         if u > min(1,A) % then reject the move:
            r_DEATH           = r_DEATH + 1;
         else % accept the move:
            
            k = k-1;
            breakpoints = breakpoints_new;
            
            [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);
                     
            log_score = COMPUTE_LOG_SCORE_OF_DAG_k(DATA_NEW, DAG,     breakpoints, lambda,m);
            a_DEATH   = a_DEATH + 1;
         end
        
         clear DATA;
         clear DATA_NEW;
            
          
    else % PERFORM A BREAKPOINT REALLOCATION MOVE
       
        % this move has probability zero if card(breakpoints) = 0;
        
        k_break = k-1;

        index = randperm(k_break);
    
        breakpoints_add = [1,breakpoints,m];
    
        k_minus = breakpoints_add(index);
        k_i     = breakpoints_add(index+1);
        k_plus  = breakpoints_add(index+2);   
        
        k_i_new = k_minus + rand(1) * (k_plus - k_minus);
        
        breakpoints_new        = breakpoints;
        breakpoints_new(index) = k_i_new;
       
     
        log_prob_ratio = log(k_i_new - k_minus) + log(k_plus - k_i_new) - log(k_i - k_minus) - log(k_plus - k_i);

         [vector]     = BREAKPOINTS_TO_VECTOR(breakpoints,m);
         [vector_new] = BREAKPOINTS_TO_VECTOR(breakpoints_new,m);
         
             for node_i=1:n_nodes
                    data = DATA_ALL{node_i};
                    for i=1:k      
                        DATA{node_i}{i}     = data(:,find(vector==i));
                        DATA_NEW{node_i}{i} = data(:,find(vector_new==i));
                    end
             end
            
          local_score_old = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA,     DAG);
          local_score_new = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA_NEW, DAG);
              
          A = exp(local_score_new - local_score_old + log_prob_ratio);
        
          u = rand(1); 
 
          if u > min(1,A) % then reject the move:
            r_ALLO           = r_ALLO + 1;
          else % accept the move:
            
           breakpoints = breakpoints_new;
           
           [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m);
                     
           log_score = COMPUTE_LOG_SCORE_OF_DAG_k(DATA_NEW, DAG,     breakpoints, lambda,m);
           a_ALLO   = a_ALLO + 1;
          end         
        
          clear DATA;
          clear DATA_NEW;
          
    end % MOVE-TYPE-LOOP
         
 
end % ITERATION-STEPS-LOOP

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [vector] = BREAKPOINTS_TO_VECTOR(breakpoints,m)

vector = ones(1,m);

if isempty(breakpoints)
    vector = ones(1,m);
else
    breakpoints_add = [1,breakpoints,m];
    
    for i=2:length(breakpoints_add)       
        vector(ceil(breakpoints_add(i-1)):floor(breakpoints_add(i))) = i-1;    
    end

end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [log_score] = COMPUTE_LOG_SCORE_OF_DAG_k(DATA, DAG, breakpoints, lambda,m)

% Interpretation of DATA:
% DATA{node_i}{1} -> data set of observations allocated to k=1 
% for scoring node_i
%
% DATA{node_i}{2} -> data set of observations allocated to k=2
% for scoring node_i
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some hyperparameters are always the same:
global T_0;
global v;
global alpha;
global my_0;
global Prior;

k = length(DATA{1});      % k is the number of mixture components

k_break = k-1;

log_prob_k = log(poisspdf(k_break,lambda));

[n_plus_thrash, n_nodes] = size(DAG);
clear n_plus_thrash;

log_prob_break = 0;

if isempty(breakpoints)   
    % log_prob_break = 0 is okay
else
    
    % (2*k_break+1)! * (1/(m-1))^{k_break}
    
    log_prob_break = log(prod(1:(2*k_break+1))) + (-1)*k_break*log(m-1);
    
    % interval [1,m] -> length = m-1
    breakpoints = [1,breakpoints,m];
    
    for i=2:length(breakpoints)
        
       log_prob_break = log_prob_break + log((breakpoints(i)-breakpoints(i-1))) - log((m-1));
        
    end
        
end

%Initialisation:
log_score = log_prob_k + log_prob_break;  

for node=1:n_nodes
    log_score = log_score + Prior(length(find(DAG(:,node)))+1);
end


% Compute the score for each of the k mixture components:
for component=1:k
    
    log_score_component = 0;
 
    [n_plus, n_obs_in_component] =  size(DATA{1}{component});

    if(n_obs_in_component==0)
        
    log_score_component = 0; 
    
    else
        
        % Compute the score for the current mixture component:
        for node = 1:n_nodes
            
            data = DATA{node}{component};
    
            % Compute T_m for the current combination of node and
            % component:
            S_m = zeros(n_plus,n_plus);
    
                for i=1:n_obs_in_component    
                S_m = S_m + (data(:,i) - mean(data')') * (data(:,i) - mean(data')')';
                end
  
            T_m = T_0 + S_m + (v*n_obs_in_component)/(v+n_obs_in_component) * (my_0 - mean(data')') * (my_0 - mean(data')')';  
            
            parents_of_node  = find(DAG(:,node));
            parents_and_node = [parents_of_node; n_plus];
           
            [log_score_local_node_nominator]   = Gauss_Score_complete(length(parents_and_node), n_obs_in_component, v, alpha, T_0(parents_and_node,parents_and_node), T_m(parents_and_node,parents_and_node));
  
            [log_score_local_node_denominator] = Gauss_Score_complete(length(parents_of_node),  n_obs_in_component, v, alpha, T_0(parents_of_node,parents_of_node),   T_m(parents_of_node,parents_of_node));
    
            log_score_component = log_score_component + (log_score_local_node_nominator - log_score_local_node_denominator);     
        end
        
    end
  
    log_score = log_score + log_score_component;
    
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [log_Score_i] = Gauss_Score_complete(n_nodes_in_sub, n_obs, v, alpha, T_0_sub, T_m_sub)

    if n_nodes_in_sub==0
    log_Score_i = 0;
    else
      
    sum_1 = (-1)*(n_nodes_in_sub)*(n_obs)/2 * log(2*pi) + (n_nodes_in_sub/2) * log(v/(v+n_obs));

    sum_3 = (alpha/2) * log(det(T_0_sub)) + (-1)*(alpha+n_obs)/2 * log(det(T_m_sub));

    log_value_a = (alpha         *n_nodes_in_sub/2)*log(2) + (n_nodes_in_sub*(n_nodes_in_sub-1)/4)*log(pi);
    log_value_b = ((alpha+n_obs) *n_nodes_in_sub/2)*log(2) + (n_nodes_in_sub*(n_nodes_in_sub-1)/4)*log(pi);
 
        for i=1:n_nodes_in_sub
            log_value_a = log_value_a + gammaln(( alpha   +1-i)/2);
            log_value_b = log_value_b + gammaln(((alpha+n_obs)+1-i)/2);
        end
   
    log_Score_i = sum_1 + (-1)*(log_value_a - log_value_b) + sum_3; 

    end

return 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [log_score] = COMPUTE_LIKELIHOOD_OF_DAG_k_ONLY(DATA, DAG)

% Interpretation of DATA:
% DATA{node_i}{1} -> data set of observations allocated to k=1 
% for scoring node_i
%
% DATA{node_i}{2} -> data set of observations allocated to k=2
% for scoring node_i
%

% The hyperparameters:
global T_0;
global v;
global alpha;
global my_0;
global Prior;


k = length(DATA{1});      % k is the number of mixture components

[n_plus_thrash, n_nodes] = size(DAG);
clear n_plus_thrash;

%Initialisation:
log_score = 0;  

for node=1:n_nodes
    log_score = log_score + Prior(length(find(DAG(:,node)))+1);
end

% Compute the score for each of the k mixture components:
for component=1:k
    
    log_score_component = 0;
 
    [n_plus, n_obs_in_component] =  size(DATA{1}{component});

    if(n_obs_in_component==0)
        
    log_score_component = 0; 
    
    else
       
        % Compute the score for the current mixture component:
        for node = 1:n_nodes
            
            data = DATA{node}{component};
    
            % Compute T_m for the current combination of node and
            % component:
            S_m = zeros(n_plus,n_plus);
    
                for i=1:n_obs_in_component    
                S_m = S_m + (data(:,i) - mean(data')') * (data(:,i) - mean(data')')';
                end
  
            T_m = T_0 + S_m + (v*n_obs_in_component)/(v+n_obs_in_component) * (my_0 - mean(data')') * (my_0 - mean(data')')';  
            
            parents_of_node  = find(DAG(:,node));
            parents_and_node = [parents_of_node; n_plus];
           
            [log_score_local_node_nominator]   = Gauss_Score_complete(length(parents_and_node), n_obs_in_component, v, alpha, T_0(parents_and_node,parents_and_node), T_m(parents_and_node,parents_and_node));
  
            [log_score_local_node_denominator] = Gauss_Score_complete(length(parents_of_node),  n_obs_in_component, v, alpha, T_0(parents_of_node,parents_of_node),   T_m(parents_of_node,parents_of_node));
    
            log_score_component = log_score_component + (log_score_local_node_nominator - log_score_local_node_denominator);     
        end
        
    end
    
    log_score = log_score + log_score_component;
  
end

return
